td.java, td.java-ln {vertical-align:top;}
tt.java, tt.java-ln, pre.java, pre.java-ln {line-height:1em; margin-bottom:0em;}
td.java-ln { text-align:right; }
tt.java-ln, pre.java-ln { color:#888888 }
/* Background       */ span.java0  { font-size: 10pt; color:#ffffff; }
/* Line numbers       */ span.java1  { font-size: 10pt; color:#808080; }
/* Multi-line comments       */ span.java2  { font-size: 10pt; color:#3f7f5f; }
/* Single-line comments       */ span.java3  { font-size: 10pt; color:#3f7f5f; }
/* Keywords       */ span.java4  { font-size: 10pt; color:#7f0055; font-weight:bold; }
/* Strings       */ span.java5  { font-size: 10pt; color:#2a00ff; }
/* Character constants       */ span.java6  { font-size: 10pt; color:#990000; }
/* Numeric constants       */ span.java7  { font-size: 10pt; color:#990000; }
/* Parenthesis       */ span.java8  { font-size: 10pt; color:#000000; }
/* Primitive Types       */ span.java9  { font-size: 10pt; color:#7f0055; font-weight:bold; }
/* Others       */ span.java10  { font-size: 10pt; color:#000000; }
/* Javadoc keywords       */ span.java11  { font-size: 10pt; color:#7f9fbf; }
/* Javadoc HTML tags       */ span.java12  { font-size: 10pt; color:#7f7f9f; }
/* Javadoc links       */ span.java13  { font-size: 10pt; color:#3f3fbf; }
/* Javadoc others       */ span.java14  { font-size: 10pt; color:#3f5fbf; }
/* Undefined       */ span.java15  { font-size: 10pt; color:#ff6100; }
/* Annotation       */ span.java16  { font-size: 10pt; color:#646464; }

div.java {
  border: 2px solid #3d95ff;
  line-height: 100%;
}

div.left {
  float: left;
  width: 48%;
}

div.right {
  float: right;
  width: 48%;
}

div.clear {
  clear: both;
}

h3.java {
  border: 0px;
  margin: 0px;
  background-color: #3d95ff;
  color: #ffffff;
  font-weight: normal;
  font-size: small;
}

code.java {
  display: block;
  overflow: hidden;
  border-left: 1em solid #ece9d8;
  line-height: 13px;
  white-space: nowrap;
}

span.onEDT {
  background-color:#c0FFa0;
}

span.offEDT {
  background-color:#FFb0c0;
}

*.centeredImage {
  text-align:center;
  margin-top:0px;
  margin-bottom:0px;
  padding:0px;
  display:block;
}

acronym {
  border-bottom: 1px dotted black;
  cursor: help;
}